part of '../loader_plus.dart';

typedef LoaderCallback<T> = Future<T> Function();
typedef WidgetValueBuilder<T> = Widget Function(BuildContext context, T value);

class Loader<T> extends StatefulWidget {
  final bool autoReload;
  final LoaderCallback<T> load;
  final WidgetValueBuilder<T>? builder;
  final Widget? loadingWidget;
  final Widget Function(Object? error)? errorBuilder;

  const Loader({
    Key? key,
    required this.load,
    this.autoReload = true,
    this.builder,
    this.loadingWidget,
    this.errorBuilder,
  }) : super(key: key);

  @override
  _LoaderState<T> createState() => _LoaderState<T>();
}

class _LoaderState<T> extends State<Loader<T>>  {


  @override
  Widget build(BuildContext context) {
    final runOnce = AsyncMemoizer<T>().runOnce(() => widget.load());
    return FutureBuilder<T>(
      future: runOnce,
      builder: (context, snapshot) {
        final data = snapshot.data;
        if (snapshot.hasData && data != null) {
          return widget.builder?.call(context, data) ??
              const SizedBox.shrink();
        } else if (snapshot.hasError) {
          return widget.errorBuilder?.call(snapshot.error) ?? Text('${snapshot.error}');
        } else if (snapshot.connectionState == ConnectionState.waiting) {
          return widget.loadingWidget ?? const SizedBox.shrink();
        }else{
          return const SizedBox.shrink();
        }
      },
    );
  }
}
